@@ -25,6 +25,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef __linux__
+#include <sys/prctl.h>
+#endif
 #include <sys/stat.h>
 #include <unistd.h>
 #include "bitmap.h"
@@ -2373,11 +2376,9 @@ xsleep(unsigned int seconds)
     ovsrcu_quiesce_end();
 }
 
-/* High resolution sleep. */
-void
-xnanosleep(uint64_t nanoseconds)
+static void
+xnanosleep__(uint64_t nanoseconds)
 {
-    ovsrcu_quiesce_start();
 #ifndef _WIN32
     int retval;
     struct timespec ts_sleep;
@@ -2405,9 +2406,37 @@ xnanosleep(uint64_t nanoseconds)
                        ovs_lasterror_to_string());
     }
 #endif
+}
+
+/* High resolution sleep with thread quiesce. */
+void
+xnanosleep(uint64_t nanoseconds)
+{
+    ovsrcu_quiesce_start();
+    xnanosleep__(nanoseconds);
     ovsrcu_quiesce_end();
 }
 
+/* High resolution sleep without thread quiesce. */
+void
+xnanosleep_no_quiesce(uint64_t nanoseconds)
+{
+    xnanosleep__(nanoseconds);
+}
+
+#if __linux__
+void
+set_timer_resolution(unsigned long nanoseconds)
+{
+    prctl(PR_SET_TIMERSLACK, nanoseconds);
+}
+#else
+void
+set_timer_resolution(unsigned long nanoseconds OVS_UNUSED)
+{
+}
+#endif
+
 /* Determine whether standard output is a tty or not. This is useful to decide
  * whether to use color output or not when --color option for utilities is set
  * to `auto`.
