@@ -1013,7 +1013,9 @@ class Idl(object):
             if not row:
                 raise error.Error('Modify non-existing row')
 
+            del table.rows[uuid]
             old_row = self.__apply_diff(table, row, row_update['modify'])
+            table.rows[uuid] = row
             return Notice(ROW_UPDATE, row, Row(self, table, uuid, old_row))
         else:
             raise error.Error('<row-update> unknown operation',
@@ -1044,9 +1046,10 @@ class Idl(object):
                 op = ROW_UPDATE
                 vlog.warn("cannot add existing row %s to table %s"
                           % (uuid, table.name))
+                del table.rows[uuid]
+
             changed |= self.__row_update(table, row, new)
-            if op == ROW_CREATE:
-                table.rows[uuid] = row
+            table.rows[uuid] = row
             if changed:
                 return Notice(ROW_CREATE, row)
         else:
@@ -1058,9 +1061,11 @@ class Idl(object):
                 # XXX rate-limit
                 vlog.warn("cannot modify missing row %s in table %s"
                           % (uuid, table.name))
+            else:
+                del table.rows[uuid]
+
             changed |= self.__row_update(table, row, new)
-            if op == ROW_CREATE:
-                table.rows[uuid] = row
+            table.rows[uuid] = row
             if changed:
                 return Notice(op, row, Row.from_json(self, table, uuid, old))
         return False
@@ -1256,7 +1261,7 @@ class Row(object):
         d["a"] = "b"
         row.mycolumn = d
 """
-    def __init__(self, idl, table, uuid, data):
+    def __init__(self, idl, table, uuid, data, persist_uuid=False):
         # All of the explicit references to self.__dict__ below are required
         # to set real attributes with invoking self.__getattr__().
         self.__dict__["uuid"] = uuid
@@ -1311,6 +1316,10 @@ class Row(object):
         # in the dictionary are all None.
         self.__dict__["_prereqs"] = {}
 
+        # Indicates if the specified 'uuid' should be used as the row uuid
+        # or let the server generate it.
+        self.__dict__["_persist_uuid"] = persist_uuid
+
     def __lt__(self, other):
         if not isinstance(other, Row):
             return NotImplemented
@@ -1849,7 +1858,11 @@ class Transaction(object):
                 op = {"table": row._table.name}
                 if row._data is None:
                     op["op"] = "insert"
-                    op["uuid-name"] = _uuid_name_from_uuid(row.uuid)
+                    if row._persist_uuid:
+                        op["uuid"] = str(row.uuid)
+                    else:
+                        op["uuid-name"] = _uuid_name_from_uuid(row.uuid)
+
                     any_updates = True
 
                     op_index = len(operations) - 1
@@ -2089,20 +2102,22 @@ class Transaction(object):
             row._mutations['_removes'].pop(column.name, None)
         row._changes[column.name] = datum.copy()
 
-    def insert(self, table, new_uuid=None):
+    def insert(self, table, new_uuid=None, persist_uuid=False):
         """Inserts and returns a new row in 'table', which must be one of the
         ovs.db.schema.TableSchema objects in the Idl's 'tables' dict.
 
         The new row is assigned a provisional UUID.  If 'uuid' is None then one
         is randomly generated; otherwise 'uuid' should specify a randomly
-        generated uuid.UUID not otherwise in use.  ovsdb-server will assign a
-        different UUID when 'txn' is committed, but the IDL will replace any
-        uses of the provisional UUID in the data to be to be committed by the
-        UUID assigned by ovsdb-server."""
+        generated uuid.UUID not otherwise in use.  If 'persist_uuid' is true
+        and 'new_uuid' is specified, IDL requests the ovsdb-server to assign
+        the same UUID, otherwise ovsdb-server will assign a different UUID when
+        'txn' is committed and the IDL will replace any uses of the provisional
+        UUID in the data to be committed by the UUID assigned by
+        ovsdb-server."""
         assert self._status == Transaction.UNCOMMITTED
         if new_uuid is None:
             new_uuid = uuid.uuid4()
-        row = Row(self.idl, table, new_uuid, None)
+        row = Row(self.idl, table, new_uuid, None, persist_uuid=persist_uuid)
         table.rows[row.uuid] = row
         self._txn_rows[row.uuid] = row
         return row
